"use strict";
/**
 * Copyright 2019 Google LLC. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const Octokit = require("@octokit/rest");
const chalk_1 = require("chalk");
const semver = require("semver");
const checkpoint_1 = require("./checkpoint");
class GitHub {
    constructor(options) {
        this.token = options.token;
        this.owner = options.owner;
        this.repo = options.repo;
        this.octokit = new Octokit({ auth: this.token });
    }
    commitsSinceSha(sha, perPage = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            var e_1, _a;
            const commits = [];
            try {
                for (var _b = __asyncValues(this.octokit.paginate.iterator({
                    method: 'GET',
                    url: `/repos/${this.owner}/${this.repo}/commits?per_page=${perPage}`
                })), _c; _c = yield _b.next(), !_c.done;) {
                    const response = _c.value;
                    for (let i = 0, commit; response.data[i] !== undefined; i++) {
                        commit = response.data[i];
                        if (commit.sha === sha) {
                            return commits;
                        }
                        else {
                            // conventional commits parser expects:
                            // [commit message]
                            // -hash-
                            // [commit sha]
                            commits.push(`${commit.commit.message}\n-hash-\n${commit.sha}`);
                        }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return commits;
        });
    }
    latestTag(perPage = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            const tags = yield this.allTags(perPage);
            const versions = Object.keys(tags);
            // TODO: we can improve the latestTag logic by using
            // this.octokit.repos.getLatestRelease as a sanity check, this should
            // help address concerns that the largest tag might be on a branch
            // other than master.
            versions.sort(semver.rcompare);
            return {
                name: tags[versions[0]].name,
                sha: tags[versions[0]].sha,
                version: tags[versions[0]].version
            };
        });
    }
    allTags(perPage = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            var e_2, _a;
            const tags = {};
            try {
                for (var _b = __asyncValues(this.octokit.paginate.iterator({
                    method: 'GET',
                    url: `/repos/${this.owner}/${this.repo}/tags?per_page=${perPage}`
                })), _c; _c = yield _b.next(), !_c.done;) {
                    const response = _c.value;
                    response.data.forEach((data) => {
                        const version = semver.valid(data.name);
                        if (version) {
                            tags[version] = { sha: data.commit.sha, name: data.name, version };
                        }
                    });
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
            return tags;
        });
    }
    addLabel(pr, label) {
        return __awaiter(this, void 0, void 0, function* () {
            checkpoint_1.checkpoint(`adding label ${chalk_1.default.green(label)} to https://github.com/${this.owner}/${this.repo}/pull/${pr}`, checkpoint_1.CheckpointType.Success);
            yield this.octokit.issues.addLabels({
                owner: this.owner,
                repo: this.repo,
                issue_number: pr,
                labels: [label]
            });
        });
    }
    openPR(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let refName = yield this.refByBranchName(options.branch);
            // If the branch exists, we delete it and create a new branch
            // with the same name; this results in the existing PR being closed.
            if (!refName) {
                refName = `refs/heads/${options.branch}`;
            }
            else {
                try {
                    checkpoint_1.checkpoint(`branch ${chalk_1.default.red(options.branch)} already exists`, checkpoint_1.CheckpointType.Failure);
                    yield this.octokit.git.deleteRef({
                        owner: this.owner,
                        repo: this.repo,
                        // DELETE requires that the refs/ prefix used during
                        // creation be removed; this seems like a bug we should log.
                        ref: refName.replace('refs/', '')
                    });
                    checkpoint_1.checkpoint(`branch ${chalk_1.default.red(options.branch)} deleted`, checkpoint_1.CheckpointType.Success);
                }
                catch (err) {
                    if (err.status === 404) {
                        // the most likely cause of a 404 during this step is actually
                        // that the user does not have access to the repo:
                        throw new AuthError();
                    }
                    else {
                        throw err;
                    }
                }
            }
            // always create the branch (it was potentially deleted in the prior step).
            try {
                checkpoint_1.checkpoint(`creating branch ${chalk_1.default.green(options.branch)}`, checkpoint_1.CheckpointType.Success);
                yield this.octokit.git.createRef({ owner: this.owner, repo: this.repo, ref: refName, sha: options.sha });
            }
            catch (err) {
                if (err.status === 404) {
                    // the most likely cause of a 404 during this step is actually
                    // that the user does not have access to the repo:
                    throw new AuthError();
                }
                else {
                    throw err;
                }
            }
            yield this.updateFiles(options.updates, options.branch, refName);
            const title = `chore: release ${options.version}`;
            checkpoint_1.checkpoint(`open pull-request: ${chalk_1.default.yellow(title)}`, checkpoint_1.CheckpointType.Success);
            const resp = yield this.octokit.pulls.create({
                owner: this.owner,
                repo: this.repo,
                title,
                head: options.branch,
                base: 'master'
            });
            return resp.data.number;
        });
    }
    updateFiles(updates, branch, refName) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < updates.length; i++) {
                const update = updates[i];
                let content;
                try {
                    content = yield this.octokit.repos.getContents({
                        owner: this.owner,
                        repo: this.repo,
                        path: update.path,
                        ref: refName
                    });
                }
                catch (err) {
                    if (err.status !== 404)
                        throw err;
                    // if the file is missing and create = false, just continue
                    // to the next update, otherwise create the file.
                    if (!update.create) {
                        checkpoint_1.checkpoint(`file ${chalk_1.default.green(update.path)} did not exist`, checkpoint_1.CheckpointType.Failure);
                        continue;
                    }
                }
                const contentText = content ?
                    Buffer.from(content.data.content, 'base64').toString('utf8') :
                    undefined;
                const updatedContent = update.updateContent(contentText);
                if (content) {
                    yield this.octokit.repos.updateFile({
                        owner: this.owner,
                        repo: this.repo,
                        path: update.path,
                        message: `updated ${update.path}`,
                        content: Buffer.from(updatedContent, 'utf8').toString('base64'),
                        sha: content.data.sha,
                        branch
                    });
                }
                else {
                    yield this.octokit.repos.createFile({
                        owner: this.owner,
                        repo: this.repo,
                        path: update.path,
                        message: `created ${update.path}`,
                        content: Buffer.from(updatedContent, 'utf8').toString('base64'),
                        branch
                    });
                }
            }
        });
    }
    refByBranchName(branch) {
        return __awaiter(this, void 0, void 0, function* () {
            var e_3, _a;
            let ref;
            try {
                try {
                    for (var _b = __asyncValues(this.octokit.paginate.iterator({ method: 'GET', url: `/repos/${this.owner}/${this.repo}/git/refs` })), _c; _c = yield _b.next(), !_c.done;) {
                        const response = _c.value;
                        for (let i = 0, r; response.data[i] !== undefined; i++) {
                            r = response.data[i];
                            const refRe = new RegExp(`/${branch}$`);
                            if (r.ref.match(refRe)) {
                                ref = r.ref;
                            }
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
            catch (err) {
                if (err.status === 404) {
                    // the most likely cause of a 404 during this step is actually
                    // that the user does not have access to the repo:
                    throw new AuthError();
                }
                else {
                    throw err;
                }
            }
            return ref;
        });
    }
}
exports.GitHub = GitHub;
class AuthError extends Error {
    constructor() {
        super('unauthorized');
        this.status = 401;
    }
}
//# sourceMappingURL=github.js.map